package board;

import events.IWinCondition;
import objects.GameObject;
import objects.Player;
import objects.Token;

import java.util.ArrayList;
import java.util.List;


public class Board implements IBoard{
    private final Grid<GameObject> grid;
    private final List<Player> players;
    private int playersTurn = 0;
    private final List<IWinCondition> conditionList;
    private boolean won = false;

    public Board(int width, int height, GameObject obj, List<IWinCondition> conditions) {
        grid = new Grid<>(width, height, obj);
        players = new ArrayList<>();
        conditionList = conditions;
    }

    @Override
    public boolean addPlayer(Player p) {
        if (!playerExists(p)) {
            players.add(p);
            return true;
        } else {
            return false;
        }
    }

    @Override
    public ArrayList<Player> getPlayers() {
        return new ArrayList<>(players);
    }

    /**
     * Gets the next player.
     *
     * @return A Player
     */
    public Player nextPlayer() {
        Player p = players.get(playersTurn);
        this.playersTurn = this.playersTurn + 2 > this.players.size() ? 0 : this.playersTurn + 1;
        return p;
    }

    /**
     * Makes sure we don't have two equal players.
     *
     * @param player    The new player to test
     * @return          True if the player is non-unique
     */
    private boolean playerExists(Player player) {
        for (Player p : players) {
            if (player.equals(p)) {
                return true;
            }
        }
        return false;
    }

    @Override
    public boolean isFull() {
        return grid.isFull();
    }

    /**
     * Checks if anyone has won.
     *
     * @param i The column of the last dropped object
     */
    private void checkWin(int i) {
        for (IWinCondition con : conditionList) {
            if (con.hasWon(i, getTopObject(i), this)) {
                won = true;
            }
        }
    }

    /**
     * Checks if anyone has won.
     *
     * @param x The x-coordinate of the last dropped object
     * @param y The y-coordinate of the last dropped object
     */
    private void checkWin(int x, int y) {
        for (IWinCondition con : conditionList) {
            if (con.hasWon(x, y, this)) {
                won = true;
            }
        }
    }

    @Override
    public boolean won() {
        return won;
    }

    @Override
    public void flush() {
        grid.flush();
        playersTurn = 0;
        won = false;
    }

    @Override
    public int getTopObject(int column) {
        return grid.getTopObject(column);
    }

    @Override
    public int getHeight() {
        return grid.getHeight();
    }

    @Override
    public int getWidth() {
        return grid.getWidth();
    }

    @Override
    public void set(int x, int y, GameObject element) {
        grid.set(x, y, element);
        if (element instanceof Token) {
            checkWin(x, y);
        }
    }

    @Override
    public boolean drop(int i, GameObject element) {
        boolean dropped = grid.drop(i, element);
        if (element instanceof Token) {
            checkWin(i);
        }
        return dropped;
    }

    @Override
    public GameObject get(int x, int y) { return grid.get(x, y); }

    @Override
    public IGrid<GameObject> copy() {
        return grid.copy();
    }
}